/* 

Functions for checking CPU firmware version with LW3
====================================================
 
Required API version: v1.2.0 

Globals: deviceIP

*/

var firmwareVersionToInt = function(version) {
    if(!version instanceof FirmwareVersionArgument) {
        throw "firmwareVersionToInt() function is called with invalid argument: version.";
    }
    
    return ((version.getMajorVersion() * 1000 + version.getMinorVersion()) * 1000 + version.getHotfixNumber()) * 1000 + version.getRcNumber();
}

var firmwareVersionStrToInt = function(fwVersionStr) {
    var matches = function(regex, str) {
        return new RegExp(regex).test(str);
    };

    var result = 0;       
   
    fwVersionStr = fwVersionStr.split(" ")[0];   
   
    if( !matches("[b,B]", fwVersionStr) ) {        
        fwVersionStr = fwVersionStr + "b0";
    }    
    
    if( matches("[v,V]\\d+\\.\d+\\.\\d+[b,B]\\d+", fwVersionStr) ||
        matches("\\d+\\.\\d+\\.\\d+[b,B]\\d+", fwVersionStr) ) {
        
        var components = fwVersionStr.split(new RegExp("[v,V|.|b,B]"));        
        
        for(i in components) {                    
            var i = parseInt(components[i], 10);
            result = result * 1000 + (isNaN(i) ? 0 : i);                     
        }
    }
    
    return result;
}

var getCpuFwVersionStr = function() {
    var lw3Connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 5);    
    var fwVersionProperty = new Property("FirmwareVersion", new Path("/SYS/MB"));
    
    var getFwVersionProperty = new GetProperty(lw3Connection, fwVersionProperty);
    getFwVersionProperty.execute();

    return fwVersionProperty.getCachedValue();    
}

var checkCpuFwVersion = function(fwVersionStr) {
    var cpuFwVersionStr = getCpuFwVersionStr();
    
    print(cpuFwVersionStr);
    
    return firmwareVersionStrToInt(fwVersionStr) <= firmwareVersionStrToInt(cpuFwVersionStr);
}

var checkCpuFwVersionRange = function(fwVersionMinStr, fwVersionMaxStr) {
    var cpuFwVersionStr = getCpuFwVersionStr();
   
    return (firmwareVersionStrToInt(fwVersionMinStr) <= firmwareVersionStrToInt(cpuFwVersionStr)) &&
           (firmwareVersionStrToInt(cpuFwVersionStr) <= firmwareVersionStrToInt(fwVersionMaxStr));
}
