/* 

Functions for getting and settings package version on legacy (UPB) devices.
===========================================================================
 
Required API version: v1.2.0 

Globals: deviceIP, packageFile

*/

var packageVersionToInt = function(version) {
    if(!version instanceof PackageVersionArgument) {
        throw "packageVersionToInt() function is called with invalid argument: version.";
    }
    
    return ((version.getMajorVersion() * 1000 + version.getMinorVersion()) * 1000 + version.getHotfixNumber()) * 1000 + version.getRcNumber();
}

var getPackageVersion = function(uidFvsEepromConfig) {
    if(!uidFvsEepromConfig instanceof I2cEepromConfig) {
        throw "getPackageVersion() function is called with invalid argument: uidFvsEepromConfig.";
    }
    
    var upbUidArgument = UpbUidArgument.newBuilder()
        .setDeviceIp(deviceIp)
        .setConfig(uidFvsEepromConfig)
        .build();

    var upbGetVersionStorageFlow = new UpbGetVersionStorageFlow(upbUidArgument);
    upbGetVersionStorageFlow.execute();

    logger.info("Package current version on device: " + upbGetVersionStorageFlow.getPackageVersionStorage().getVersionAsString() );
    
    try {
        return upbGetVersionStorageFlow.getPackageVersionStorage().toPackageVersionArgument();
        
    } catch(exception){
        return null;
    }
}

var setPackageVersion = function(uidFvsEepromConfig) {
    if(!uidFvsEepromConfig instanceof I2cEepromConfig) {
        throw "setPackageVersion() function is called with invalid argument: uidFvsEepromConfig.";
    }    
    
    var upbSetPackageVersionArgument = UpbSetPackageVersionArgument.newBuilder()
        .setUidArgument(UpbUidArgument.newBuilder()
            .setDeviceIp(deviceIp)
            .setConfig(uidFvsEepromConfig)
            .build())
        .setPackageVersionStorage(PackageVersionStorage.newBuilder()
            .set(packageFile.getPackageVersionArgument())
            .build())
        .build();

    var upbSetPackageVersionFlow = new UpbSetPackageVersionFlow(upbSetPackageVersionArgument);
    upbSetPackageVersionFlow.execute();
}

var isCurrentPackageNewer = function(uidFvsEepromConfig) {    
    if(!uidFvsEepromConfig instanceof I2cEepromConfig) {
        throw "isCurrentPackageNewer() function is called with invalid argument: uidFvsEepromConfig.";
    }
    
    var uidArgument = UpbUidArgument.newBuilder()
        .setDeviceIp(deviceIp)
        .setConfig(uidFvsEepromConfig)
        .build();
        
    var getVersionStorage = new UpbGetVersionStorageFlow(uidArgument);
    getVersionStorage.execute();
    
    try {
        var versionOnDevice = getVersionStorage.getPackageVersionStorage().toPackageVersionArgument();
        var versionInPackage = packageFile.getPackageVersionArgument();
        
        return packageVersionToInt(versionInPackage) < packageVersionToInt(versionOnDevice);
        
    } catch(exception){
        return true;
    }
}