/* 

Functions for transferring files to/from legacy (UPB) devices.
==============================================================
 
Required API version: v1.2.0 

Globals: deviceIP, isGui

*/

/* The method which does the update process including tftp communication, updating progress indicator and updating the checkpoint. */
var uploadFileWithTftp = function(tftpFile, parentCheckpoint, coeffParam) {
    if(!(tftpFile instanceof InternalFlashFile) &&
        !(tftpFile instanceof ExternalSpiFlashFile) &&
        !(tftpFile instanceof I2cEepromFile)) {
            
        throw "uploadFileWithTftp() function is called with invalid argument: tftpFile.";
    }
    if(!parentCheckpoint instanceof Checkpoint) {
        throw "uploadFileWithTftp() function is called with invalid argument: parentCheckpoint.";
    }
    
    var upbPutFileFlow = UpbPutFileFlow.newBuilder()
        .setGenericTftpFile(tftpFile)
        .build();

    /* Get the tftpPutFlow object. The object is needed to subscribe on it's progress by progressIndicator and checkpoint  */
    var tftpPutFlow = upbPutFileFlow.getTftpPutFlow();

    /* Create a new checkpoint under containingCheckpoint */
    var tftpCheckpoint = parentCheckpoint.add("Uploading " + tftpFile.getDescription() + "...", coeffParam);

    /* If not Gui application create a progressbar */
    if(!isGui){
        var progressIndicator = new ProgressIndicator();
    }

    /* Checkpoint and progress indicator consumers subscribes to the progress of tftp file upload and verification. */
    tftpPutFlow.subscribeToProgress(
        function(progress) {
            /* Progress goes from 0.0 to 1.0 (upload and verification)*/
            tftpCheckpoint.setProgress((progress.uploadedBytes + progress.downloadedBytes) / (progress.totalBytes * 2));
            if(!isGui) {
                /* Progress goes from 0.0 to 100.0 (upload and verification) */
                progressIndicator.setProgress(parseInt(100 * (progress.uploadedBytes + progress.downloadedBytes) / (progress.totalBytes * 2)));
            }
        }
    );

    /* Do the tftp update and verification.*/
    upbPutFileFlow.execute();
    tftpCheckpoint.setProgress(1.0);
}
