include("lib/legacy/control.js")
include("lib/legacy/tftp.js")
include("lib/legacy/package.js")
include("lib/legacy/uid.js")

include("lib/backup/backup.js")

include("lib/check/checkFwVersion.js")

/* Device specific declarations */
include("tps-tx200.js")

/* Create a logger */
var logger = LoggerFactory.getLogger("main.js");

var printScriptParams = function() {
    var params = "Script parameters:\n";

    params += ("\t deviceIp: \t" + deviceIp.getHostAddress() + "\n");

    params += ("\t forceNoBackup: \t" + forceNoBackup + "\n");
    params += ("\t factoryDefault: \t" + factoryDefault + "\n");
    params += ("\t restoreFromBackup: \t" + restoreFromBackup + "\n");
    params += ("\t backupFolder: \t" + backupFolder + "\n");

    params += ("\t skipPresetsAtRestore: \t" + skipPresetsAtRestore + "\n");
    
    params += ("\t uploadDefaultMiniWeb: \t" + uploadDefaultMiniWeb + "\n");

    logger.debug(params);
}

var waitForVS100Boot = function() {
     var connection = new TcpConnection(deviceIp, 6107, 1000, 1000, 3);
     for (var i = 0; i < 20; i++) {
        try {
            var vs100VerGetProperty = new GetProperty(connection, new Property("LocalFwVersion", new Path("/REMOTE/D1")));
            vs100VerGetProperty.execute();
            var vs100Ver = vs100VerGetProperty.getProperty().getCachedValue();
            var pattern = /[1-9]/;
            if((vs100Ver.match(pattern)) !== null) {
                return;
            }
        } catch(exception){
            logger.trace("LocalFwVersion property is not available yet.");
        } finally {
            rootCheckpoint.waitMillisLong(1000);
        }
     }
     throw "Could not connect to device after starting application.";
}

/* main function */
var main = function() {  
    printScriptParams();

    rootCheckpoint.indeterminate();

    logger.info("Package version: " + packageFile.getPackageVersionArgument().getMajorVersion() + "." + packageFile.getPackageVersionArgument().getMinorVersion() + "." + packageFile.getPackageVersionArgument().getHotfixNumber() + "b" + packageFile.getPackageVersionArgument().getRcNumber());

    /* Create backup files */
    if(!forceNoBackup) {
        if(checkCpuFwVersion("v1.1.0b4")) {
            var backups = createBackup("tps_tx200.js", skipPresetsAtRestore, 10000, 90000);        
        }       
        else {
            var msg = "Creating backup of device settings is not supported by the current firmware.";
            if(!factoryDefault) {
                throw msg + " Please select factory default settings from the parameters dialog.";
            }
            else {
                logger.warn(msg + " No backup wil be created.");
            }
        }
    }
    
    /* Switch into bootload mode */
    var checkpointSwitchToBootload = rootCheckpoint.add("Switching device into bootload mode...", 0.1);
    enterBootload();
    checkpointSwitchToBootload.setProgress(1.0);

    /* Get UID information from the device */
    var checkpointGetUID = rootCheckpoint.add("Getting UID from device...", 0.1);
    var uid = getUID(uidFvsEepromConfig);
    checkpointGetUID.setProgress(1.0);

    /* Get Package version from the device */
    getPackageVersion(uidFvsEepromConfig);

    var checkpointUploadFiles = rootCheckpoint.add("Uploading and verifying files...", 0.50);
    var filesToUpload = [];       
    
    /* Add Application fw */        
    filesToUpload.push(applicationFile_HDMI_TPS_TX200);
    
    /* Check if the current firmware on the device is newer then the one in the package */
    if(!factoryDefault && (isCurrentFirmwareNewer(applicationFile_HDMI_TPS_TX200) ||
        isCurrentPackageNewer(uidFvsEepromConfig))) {
            
        logger.warn("The firmware is newer on the device, it is not possible to restore settings, all settings will be set to factory default values.");
        factoryDefault = true;
    }              
    
    /* Add Valens VS100TX fw */
    filesToUpload.push(vs100TxFile_ALL_TPS_FAMILY_PRODUCTS);
    
    /* Add PS171 (DisplayPort chip) fw */
    if( (PN_SW4_TPS_TX240 == uid.getProductPartNumber()) || 
        (PN_SW4_TPS_TX240_PLUS == uid.getProductPartNumber()) ||     
        (PN_DP_TPS_TX220 == uid.getProductPartNumber()) || 
        (PN_DP_TPS_TX210 == uid.getProductPartNumber()) ) {
            
        filesToUpload.push(ps171FirmwareFile_HDMI_TPS_TX200);
    }    

    /* Upload files */
    for (var i = 0; i < filesToUpload.length; i++) {
        uploadFileWithTftp(filesToUpload[i], checkpointUploadFiles, 1.0 / filesToUpload.length);
    }

    /* Set package version */
    var checkpointUpdatePackageVersion = rootCheckpoint.add("Updating package version...", 0.05);
    setPackageVersion(uidFvsEepromConfig);
    checkpointUpdatePackageVersion.setProgress(1.0);

    /* Start application */
    var checkpointStartApplication = rootCheckpoint.add("Starting application...", 0.1);
    startApplication(backups && restoreFromBackup && !factoryDefault, 10000, 90000);
    
    /* Due to EEPROM emulation the restore process can only be started if the VS100 is booted. */
    if(!factoryDefault) {
        logger.info("Waiting for VS100 booting...");
        waitForVS100Boot();
    }
    
    checkpointStartApplication.setProgress(1.0);

    /* Restore settings */
    if(!factoryDefault && backups && restoreFromBackup) {
        var checkpointRestoreSettings = rootCheckpoint.add("Restoring configuration settings...", 0.05);
        logger.info(checkpointRestoreSettings.getTitle());
        restoreSettings(backups['backupFilePath']);
        checkpointRestoreSettings.setProgress(1.0);
        
    } else {
        /* do nothing but increase rootCheckpoint by 5% */
        var checkpointDummy = rootCheckpoint.add("", 0.05);
        checkpointDummy.setProgress(1.0);
    }

    /* Restore presets */
    if(!factoryDefault && backups && restoreFromBackup && !skipPresetsAtRestore) {
        var presetRestoreCheckpoint = rootCheckpoint.add("Restoring device presets...", 0.05);
        logger.info(presetRestoreCheckpoint.getTitle());        
        restoreFiles(backups['presetFileMetaDataList'], 20000, presetRestoreCheckpoint);        
        presetRestoreCheckpoint.setProgress(1.0);
        
    } else {
        /* do nothing but increase rootCheckpoint by 5% */
        var checkpointDummy = rootCheckpoint.add("", 0.05);
        checkpointDummy.setProgress(1.0);
    }
    
    /* Restore miniweb file */
    if(!factoryDefault) {        
        uploadDefaultMiniWeb = uploadDefaultMiniWeb && (0 == backups['indexFileMetaDataList'].length);
        
        if(uploadDefaultMiniWeb) {            
            var checkpointUploadDefaultControlHtml = rootCheckpoint.add("Uploading built-in miniweb...", 0.05);

            logger.info(checkpointUploadDefaultControlHtml.getTitle());            
            
            var httpProtocolConnection = new HttpProtocolConnection(HttpProtocolConnectionArguments
                    .newBuilder()
                    .setUri("http://" + deviceIp.getHostAddress() + "/index.html")
                    .setSocketTimeout(60000)
                    .build());                    
           
            var httpArguments = HttpArguments.newBuilder()
                    .setInputStreamProvider(packageFile.getFileHandler("index-min.html"))                    
                    .setPostName("index")
                    .setUploadFileName("index.html")
                    .build();                    

            var httpUpload = new HttpUploadFlow(httpProtocolConnection, httpArguments);            
            httpUpload.execute();
    
            checkpointUploadDefaultControlHtml.setProgress(1.0);            
            
        } else {
            if(backups && restoreFromBackup) {
                var checkpointRestoreIndexHtml = rootCheckpoint.add("Restoring built-in miniweb file...", 0.05);
                if(0 != backups['indexFileMetaDataList'].length) {                
                    restoreFiles(backups['indexFileMetaDataList'], 60000, checkpointRestoreIndexHtml);
                }
                checkpointRestoreIndexHtml.setProgress(1.0);
            } else {
                /* do nothing but increase rootCheckpoint by 5% */
                var checkpointDummy = rootCheckpoint.add("", 0.05);
                checkpointDummy.setProgress(1.0);
            }                
        }
    } else {
        /* do nothing but increase rootCheckpoint by 5% */
        var checkpointDummy = rootCheckpoint.add("", 0.05);
        checkpointDummy.setProgress(1.0);
    }
    
    logger.trace(rootCheckpoint.getTrace());
}