var logger = LoggerFactory.getLogger("pre.js");

Object.defineProperty(this, 'SCRIPT_API_MIN_VERSION', {value: '1.2.0'});
Object.defineProperty(this, 'LDU_MIN_VERSION', {value: '1.2.3'});

addSupportedPartNumbers(
    /*SW4-TPS-TX240*/	            91540007,
    /*SW4-TPS-TX240-PLUS*/          91540065,
    /*DVI-HDCP-TPS-TX220*/	        91540010,
    /*DVI-HDCP-TPS-TX210*/	        91540015,
    /*HDMI-TPS-TX220*/	            91540011,
    /*HDMI-TPS-TX210*/	            91540017,
    /*DP-TPS-TX220*/	            91540018,
    /*DP-TPS-TX210*/	            91540019
);

mapCpuFileNameToPartNumbers("tps_tx200.bin", 91540007, 91540065, 91540010, 91540015, 91540011, 91540017, 91540018, 91540019);

var params = new Array();

var validateParams = function() {

    /* factoryDefault*/
    params.push("factoryDefault");
    if(typeof factoryDefault === 'undefined') {
        factoryDefault = false;

    } else if(typeof factoryDefault === "string") {
        if(factoryDefault != "true" && factoryDefault!= "false") {
            return false;
        }
        if(factoryDefault === "true") {
           factoryDefault = true;
        } else {
           factoryDefault = false;
        }
    }

    /* forceNoBackup*/
     params.push("forceNoBackup");
     if(typeof forceNoBackup === 'undefined') {
        forceNoBackup = false;

     } else if(typeof forceNoBackup === "string") {
         if(forceNoBackup != "true" && forceNoBackup!= "false") {
             return false;
         }
         if(forceNoBackup === "true") {
            forceNoBackup = true;
         } else{
            forceNoBackup= false;
         }
     }

    /* restoreFromBackup */
    params.push("restoreFromBackup");
    if(typeof restoreFromBackup === 'undefined') {
        if(forceNoBackup === false && factoryDefault === false) {
            restoreFromBackup = true;
        } else {
            restoreFromBackup = false;
        }
    } else if (typeof restoreFromBackup === "string") {
        if(restoreFromBackup != "true" && restoreFromBackup != "false") {
            return false;
        }
        if(restoreFromBackup === "true") {
           restoreFromBackup = true;
        } else{
           restoreFromBackup = false;
        }
    }

    if(forceNoBackup === true && restoreFromBackup === true) {
        return false;
    }

    if(factoryDefault === true && restoreFromBackup === true) {
        return false;
    }
    
    /* skipPresetsAtRestore */
    params.push("skipPresetsAtRestore");
    if(typeof skipPresetsAtRestore === "string") {
        if(skipPresetsAtRestore != "true" && skipPresetsAtRestore!= "false") {
            return false;
        }
       if(skipPresetsAtRestore === "true") {
           skipPresetsAtRestore = true;
        } else{
           skipPresetsAtRestore = false;
        }
    }
    else {
        skipPresetsAtRestore = false;
    }
    
    /* uploadDefaultMiniWeb */
    params.push("uploadDefaultMiniWeb");
    if(typeof uploadDefaultMiniWeb === 'undefined') {
        uploadDefaultMiniWeb = false;

    } else if(typeof uploadDefaultMiniWeb === "string") {
        if(uploadDefaultMiniWeb != "true" && uploadDefaultMiniWeb!= "false") {
            return false;
        }
        if(uploadDefaultMiniWeb === "true") {
           uploadDefaultMiniWeb = true;
        } else{
           uploadDefaultMiniWeb= false;
        }
    }

    return allParamsDefined(params);
}

var allParamsDefined = function(params) {
    for(var i = 0; i < params.length; i++) {
        if(typeof this[params[i]]=="undefined"){
			return false;
        }
    }
    return true;
}

var printParams = function() {
    print("Backup and restore options:\n");
    print("\tfactoryDefault:\t if set to true device will reset to factory default settings after the upgrade process is finished, default: false");
    print("\trestoreFromBackup:\t if true, no device configuration is restored from the backup created at the begininning of upgrade process (default: true)");
    print("\tskipPresetsAtRestore:\t If true. Device presets will be skipped at restore. (default: false)");
    print("\tbackupFolder:\t folder to create configuration backup into");

    print("\nAdvanced backup and restore options:\n");
    print("\tforceNoBackup:\t if true, no backup file will be created (default: false)");    
    print("\tuploadDefaultMiniWeb: If true, the default built-in miniweb will be uploaded to the device. (default: false)");    
}

var swVersionStrToInt = function(swVersionStr) {
    var matches = function(regex, str) {
        return new RegExp(regex).test(str);
    };

    var result = 0;         
   
    if( !matches("[b,B]", swVersionStr) ) {        
        swVersionStr = swVersionStr + "b0";
    }    
    
    if( matches("\\d+\\.\\d+\\.\\d+[b,B]\\d+", swVersionStr) ) {
        
        var components = swVersionStr.split(new RegExp("[v,V|.|b,B]"));        
        
        for(i in components) {                    
            result = result * 1000 + components[i];                     
        }
    }
    
    return result;
}

var checkLdu2Version = function(swVersionStr) {
    return (typeof LDU2_VERSION !== 'undefined') && (swVersionStrToInt(swVersionStr) <= swVersionStrToInt(LDU2_VERSION));
}

var excludeUnsupportedLdu2Versions = function() {
    var contentText = "The selected package requires newer version of LDU2 (min: v" + LDU_MIN_VERSION + "). Please find the latest version of LDU2 at www.lightware.com";

    if ( (SCRIPT_API_VERSION.localeCompare("1.0.0b1") == 0) || (SCRIPT_API_VERSION.localeCompare("1.1.0b1") == 0) ) {                        
        
        try {        
            var alertPopupShow = new AlertPopupShow(AlertPopupArgument.newBuilder()
                                    .setAlertType(AlertPopup.AlertType.ERROR)
                                    .setContentText(contentText)
                                    .build());

            alertPopupShow.execute();
        } catch (exception) {
            logger.debug("Showing popup is not available in CLI mode.");
        }
    }
    else {            
        if(checkLdu2Version(LDU_MIN_VERSION) ) {            
            return;
        }
    }

    throw contentText;    
}

excludeUnsupportedLdu2Versions();