/************************************
 * Configuring UID (FVS) Eeprom
 ************************************/
/* create UID EEPROM/SDA GpioCnfig */
var SDA_UID = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOB)
    .setPin(GpioConfig.GPIO_PIN_10)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_OPENDRAIN)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_2MHZ)
    .build();

/* create UID EEPROM/SCL GpioCnfig */
var SCL_UID = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOE)
    .setPin(GpioConfig.GPIO_PIN_15)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_OPENDRAIN)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_2MHZ)
    .build();

/* define UID Eeprom i2c address */
var UID_EEPROM_I2C_ADDRESS = 0xA0;

/* create UID Eeprom I2C Eeprom config */
var uidFvsEepromConfig = I2cEepromConfig.newBuilder()
    .setScl(SCL_UID)
    .setSda(SDA_UID)
    .setI2cAddress(UID_EEPROM_I2C_ADDRESS)
    .setEepromType(I2cEepromConfig.EepromType.EEPROM_24LC512K)
    .build();

/************************************
 * Configuring HDMI-TPS-TX200 series PS171's external SPI Flash
 ************************************/
/* create PS171's SPI Chip Select~ GpioConfig for HDMI-TPS-TX200 series devices */
var CSN_PS171_HDMI_TPS_TX200 = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOC)
    .setPin(GpioConfig.GPIO_PIN_11)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_50MHZ)
    .build();

/* create PS171's SPI SCLK GpioConfig for HDMI-TPS-TX200 series devices */
var SCLK_PS171_HDMI_TPS_TX200 = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOC)
    .setPin(GpioConfig.GPIO_PIN_12)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_50MHZ)
    .build();

/* create PS171's SPI MOSI GpioConfig for HDMI-TPS-TX200 series devices */
var MOSI_PS171_HDMI_TPS_TX200 = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOD)
    .setPin(GpioConfig.GPIO_PIN_0)
    .setGpioMode(GpioConfig.GpioConfig_MODE_OUTPUT)
    .setOutputMode(GpioConfig.GpioConfig_OUTPUTMODE_PUSHPULL)
    .setPullMode(GpioConfig.GpioConfig_PULLMODE_NOPULL)
    .setSpeed(GpioConfig.GpioConfig_SPEED_50MHZ)
    .build();

/* create PS171's SPI MISO GpioConfig for HDMI-TPS-TX200 series devices */
var MISO_PS171_HDMI_TPS_TX200 = GpioConfig.newBuilder()
    .setPort(GpioConfig.GPIOC)
    .setPin(GpioConfig.GPIO_PIN_10)
    .setGpioMode(GpioConfig.GpioConfig_MODE_INPUT)
    .build();

/* create External Spi Flash Config for PS171 chip for HDMI-TPS-TX200 series devices */
var ps171ExternalSpiFlashConfig_HDMI_TPS_TX200 = ExternalSpiFlashConfig.newBuilder()
    .setCsnPin(CSN_PS171_HDMI_TPS_TX200)
    .setSclkPin(SCLK_PS171_HDMI_TPS_TX200)
    .setMosiPin(MOSI_PS171_HDMI_TPS_TX200)
    .setMisoPin(MISO_PS171_HDMI_TPS_TX200)
    .setFlashType(ExternalSpiFlashConfig.FlashType.SST25VF020B)
    .build();


/************************************
 * Configuring FW files
 ************************************/

/* create Application file (Internal Flash File) for HDMI-TPS-TX200 series devices */
var applicationFile_HDMI_TPS_TX200 = InternalFlashFile.newBuilder()
    .setDeviceIp(deviceIp)
    .setBaseAddress(0x0800C000)
    .setAccessMode(InternalFlashFile.FILEACCESS_TFTP_READWRITE)
    .setPackageFile(packageFile)
    .setFileNameInPackage("tps_tx200.bin")
    .setFvsBlockNumber(0)
    .setFvsEepromConfig(uidFvsEepromConfig)
    .setDescription("HDMI-TPS-TX200 series application fw")
    .build();

/* create VS100TX file (Internal Flash File) for all TPS Family devices */
var vs100TxFile_ALL_TPS_FAMILY_PRODUCTS = InternalFlashFile.newBuilder()
   .setDeviceIp(deviceIp)
   .setBaseAddress(0x080E0000)
   .setAccessMode(InternalFlashFile.FILEACCESS_TFTP_READWRITE)
   .setPackageFile(packageFile)
   .setFileNameInPackage("vs100_tx.bin")
   .setFvsBlockNumber(1)
   .setFvsEepromConfig(uidFvsEepromConfig)
   .setDescription("VS100TX fw")
   .build();

/* create PS171 fw file (External Spi Flash File) for HDMI-TPS-TX200 series devices */
var ps171FirmwareFile_HDMI_TPS_TX200 = ExternalSpiFlashFile.newBuilder()
   .setDeviceIp(deviceIp)
   .setExternalSpiFlashConfig(ps171ExternalSpiFlashConfig_HDMI_TPS_TX200)
   .setBaseAddress(0x0)
   .setPackageFile(packageFile)
   .setFileNameInPackage("tps_family_ps171.bin")
   .setFvsBlockNumber(4)
   .setFvsEepromConfig(uidFvsEepromConfig)
   .setDescription("PS171 FW")
   .build();

/************************************
 * Part numbers of DP devices
 ************************************/
   
var PN_SW4_TPS_TX240 = 91540007;
var PN_SW4_TPS_TX240_PLUS = 91540065;
var PN_DP_TPS_TX220 = 91540018;
var PN_DP_TPS_TX210 = 91540019;
   