/**
deletionScript
var SampleDeletionScript = [
    //removed KM layer
    {version: "1.1.0b0", regex: /\/MEDIA\/KM(.*)$/g },
    //removed /MEDIA/AUDIO/I1, /MEDIA/AUDIO/I2, /MEDIA/AUDIO/I3 and /MEDIA/AUDIO/I4 ports
    {version: "1.1.0b0", regex: /\/MEDIA\/AUDIO\/I[1-4](.*)$/g },
];
 */ 
function EventManagerConverter(parser, version, revision, partNumber, deletionScript) {
    parser.convertLines(function(line) {
        if(line.node.startsWith('/EVENTS/')) {
            if(line != null && line.node.startsWith('/EVENTS/') && ((line.prop=='Condition') || (line.prop=='Action'))) {
                if(line.value.startsWith("/")) {
                    if(deletionScript != null) {
                        for(var i = 0; i < deletionScript.length; i++){
                            if(isConversionNeeded(version, deletionScript[i].version)){
                                deletionScript[i].regex.lastIndex = 0; //JavaScript RegExp object is stateful, so need to reset it manually
                                var match = deletionScript[i].regex.exec(line.value);
                                if(match != null) {
                                    parser.warnings.push( {data:line.printWithoutCommand(), msg:"Can't convert event, it will be lost"});
                                    return null;
                                }
                            }
                        }
                    }
                    var ret = convert(line.value, version, revision, partNumber );
                    line.value = new Parser(ret.restore).lines[0].printWithoutCommand(true); 
                }
            }
        }
        return line;
    });
    return parser;
};

    /* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
/* global LW3line */

//only run this, if the backup contains the EM v1
function EventManagerv1to2(parser, actualVersion, v2version) {
    if(!isConversionNeeded(actualVersion, v2version)) {
        return parser;
    }
    //collect events
    var events = new Array();
    parser.convertLines(function(line) {
        if(line.node.startsWith('/EVENTS/')) {
            if(events[line.node] == null) {
                events[line.node] = new Array();
            }
            events[line.node][line.prop] = line.value;
            return null;
        }
        return line;
    });
    for(var event in events) {
        parser.addLine(LW3line.method(event, "Reset", ""));
        var enabled = false;
        if(events[event].SourceNode && events[event].SourceProperty) {
            var value = LW3line.property(events[event].SourceNode, events[event].SourceProperty, events[event].SourcePropertyValue).printWithoutCommand(true);
            parser.addLine(LW3line.property(event, "Condition", value));
            enabled = true;
        }
        if(events[event].TargetNode && events[event].TargetProperty) {
            var value = LW3line.property(events[event].TargetNode, events[event].TargetProperty, events[event].TargetPropertyValue).printWithoutCommand(true);
            parser.addLine(LW3line.property(event, "Action", value));
            enabled = true;
        }
        if(enabled) {
            parser.addLine(LW3line.property(event, "Enabled", "true"));
        }
        if(events[event].Name) {
            parser.addLine(LW3line.property(event, "Name", events[event].Name));
        }
    }
    return parser;
};
/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

// WARNING: version numbers must be in ascending order, the code won't check it for you!
/*
 * in search null means don't care
 * in replace null means don't change
var SampleConversionScript = [
    //remove node
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: null},
    //move node
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: LW3line.property("/SYS/MB", null, null)},
    //move and rename property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: LW3line.property("/SYS/MB", null, null)},

    //remove property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: null},
    //move property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: LW3line.property("/SYS/MB", null, null)},
    //move and rename property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: LW3line.property("/SYS/MB", "ControlLock", null)},
];
 */
function SimpleScriptConvert(convert) {
    this.paramPatern = /\{([\d]+)\}/g;
    
    this.parseParam = function(search, replace, pattern) {
        var ret = Array();
        if(search == null) {
            ret.search_matches = ["{0}"];
            ret.search = new RegExp(pattern + "$", 'g');
        } else {
            this.paramPatern.lastIndex = 0;
            ret.search_matches = search.match(this.paramPatern);
            if(ret.search_matches == null) {
                ret.search_matches = ["{0}"];
            } else {
                ret.search_matches.unshift("{0}");
            }
            this.paramPatern.lastIndex = 0;
            ret.search = new RegExp(search.replace(this.paramPatern, pattern) + "$", 'g');
        }
        ret.replace = replace;
        return ret;
    };
    
    this.convertParam = function(matches, params) {
        if(params.replace == null) {
            return null;
        }
        var ret = params.replace;
        if(matches.length > 0) {
            for(var i = 0; i < matches.length; i++) {
                ret = ret.replace( params.search_matches[i], matches[i]);
            }
        }
        return ret;
    };
    this.getParamMatch = function(params, original) {
        if(original == null) {  //EM compatibility
            return new Array();
        }
        params.search.lastIndex = 0;
        if(!params.search.test(original)) {
            return null;
        }
        params.search.lastIndex = 0;
        var match = params.search.exec(original);
        if(match == null) {
            match = new Array();
        }
        match[0] = original;
        return match;
    };
    
    this.convertLine = function(line) {
        var node = this.getParamMatch(this.node, line.node);
        if(node == null) {
            return line;
        }
        var prop = this.getParamMatch(this.prop, line.prop);
        if(prop == null) {
            return line;
        }
        var value = this.getParamMatch(this.value, line.value);
        if(value == null) {
            return line;
        }
        var type = this.getParamMatch(this.type, line.type);
        if(type == null) {
            return line;
        }
        
        line.node = this.convertParam(node, this.node);
        line.prop = this.convertParam(prop, this.prop);
        line.value = this.convertParam(value, this.value);
        line.type = this.convertParam(type, this.type);
        if(line.node == null || line.prop == null || line.value == null || line.type == null) {
            return null;
        } else {
            return line;
        }
    };
    
    this.replaceReplacer = function(param) {
        return (param==null)?"{0}":param;
    };
    
    var replace;
    if(convert.replace != null) {
        replace =  {
            node:this.replaceReplacer(convert.replace.node),
            prop:this.replaceReplacer(convert.replace.prop),
            value:this.replaceReplacer(convert.replace.value),
            type:this.replaceReplacer(convert.replace.type)
        };
    } else {
        replace = {
            node:null, 
            prop:null, 
            value:null, 
            type:null
        };
    }
    this.node = this.parseParam(convert.search.node, replace.node, "([a-zA-Z0-9\\\/]*)");
    this.prop = this.parseParam(convert.search.prop, replace.prop, "([a-zA-Z0-9]*)");
    this.value = this.parseParam(convert.search.value, replace.value, "(.*)");
    this.type = this.parseParam(convert.search.type, replace.type, "(.*)");
};


function SimpleScriptConverter(parser, version, conversionScript){
    // Find the first version number to be used
    var firstPosition = -1;
    var actualVersionNumber = versionToNumber(version);
    for(var i = 0; i < conversionScript.length; i++){
        if(versionToNumber(conversionScript[i].version) > actualVersionNumber){
            firstPosition = i;
            break;
        }
    }
    // All versions in the script are older than the current version, 
    // no conversion is needed
    if(firstPosition === -1)
        return parser;
    
    // Do the conversion line-by-line
    for(var scriptIndex = firstPosition; scriptIndex < conversionScript.length; scriptIndex++){
        var converter = new SimpleScriptConvert(conversionScript[scriptIndex]);
//        parser.convertLines(converter.convertLine);
        parser.convertLines(function(line) {
            return converter.convertLine(line);
        });
    }
    
    return parser;
}



function LW3line(node, prop, value, type) {
    if(node === undefined) {
        this.node=null;
    } else {
        this.node=node;
    }
    if(prop === undefined) {
        this.prop=null;
    } else {
        this.prop=prop;
    }
    if(value === undefined) {
        this.value=null;
    } else {
        this.value=value;
    }
    if(type === undefined) {
        this.type=null;
    } else {
        this.type=type;
    }
    this.isProperty = function() {
        return this.type === "prop";
    };
    
    this.isMethod = function() {
        return this.type === "meth";
    };
    
    this.print = function(isProp) {
        if(isProp === undefined) {
            isProp = this.isProperty();
        }
        if(isProp) {
            return 'SET ' + this.printWithoutCommand(isProp);
        } else {
            return 'CALL ' + this.printWithoutCommand(isProp);
        }
    };
    this.printWithoutCommand = function(isProp) {
        if(isProp === undefined) {
            isProp = this.isProperty();
        }
        if(isProp) {
            return this.node + '.' + this.prop + '=' + this.value;
        } else {
            return this.node + ':' + this.prop + '(' + this.value + ')';
        }
    };
};

LW3line.property = function(node, prop, value) {
    return new LW3line(node, prop, value, "prop");
};

LW3line.method = function(node, prop, value) {
    return new LW3line(node, prop, value, "meth");
};

LW3line.parse = function(line) {
    var prop = /SET (\/[a-zA-Z0-9\/]*)\.([a-zA-Z0-9]*)=(.*)$/g;
    var meth = /CALL (\/[a-zA-Z0-9\/]*)\:([a-zA-Z0-9]*)\((.*)\)$/g;
    var eventmanagercompatibility = /(\/[a-zA-Z0-9\/]*)\.([a-zA-Z0-9]*)=(.*)$/g;
    var match = prop.exec(line);
    if(match != null) {
        return LW3line.property(match[1], match[2], match[3]);
    }
    match = meth.exec(line);
    if(match != null) {
        return LW3line.method(match[1], match[2], match[3]);
    }
    match = eventmanagercompatibility.exec(line);
    if(match != null) {
        return new LW3line(match[1], match[2], match[3]);
    }
    return null;
};

function Parser(data) {
    var output = "";
    this.lines = new Array();
    this.warnings = new Array();
    data = data.split('\r').join('\n'); 
    var tmp = data.split('\n');
    for(var lineIndex in tmp) {
        var line = tmp[lineIndex];
        if (line.length > 0) {
            if (!line.startsWith(';')) {
                var m = LW3line.parse(line);
                if(m != null) {
                    this.lines.push(m);
                } else {
                    throw "Parse error: " + line;
                }
            } else {
                output = output + line + "\r\n"; 
            }
        }
    }
    
    this.convertLines = function(convertFunc) {
        var newLines = new Array();
        for(var line in this.lines) {
            var newLine = convertFunc(this.lines[line], this);
            if(newLine != null) {
                newLines.push(newLine);
            }
        }
        this.lines = newLines;
    };
    this.addLine = function(line) {
        this.lines.push(line);
    };
    this.putWarning = function(line, warn) {
        warnings.push({data:line.print(), msg:warn});
    };
    this.printLines = function() {
        for(var lineNum in this.lines) {
            output = output + this.lines[lineNum].print() + "\r\n"; 
        }
        return output;
    };
    this.output = function() {
        var ret = {
            restore: this.printLines()
        };
        if(this.warnings.length>0) {
            ret.warning = this.printWarnings();
        }
        return ret;
    };
    this.printWarnings = function() {
        var output = "";
        for(var lineNum in this.warnings) {
            output = output + this.warnings[lineNum].data + ": " + this.warnings[lineNum].msg + "\r\n"; 
        }
        return output;
    };
};


/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

// WARNING: version numbers must be in ascending order, the code won't check it for you!
/*
 * in search null means don't care
 * in replace null means don't change
var SampleConversionScript = [
    //remove node
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: null},
    //move node
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: LW3line.property("/SYS/MB", null, null)},
    //move and rename property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", null, null) , replace: LW3line.property("/SYS/MB", null, null)},

    //remove property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: null},
    //move property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: LW3line.property("/SYS/MB", null, null)},
    //move and rename property
    {version: "1.1.0b0", search: LW3line.property("/SYS/CORE", "ButtonLock", null) , replace: LW3line.property("/SYS/MB", "ControlLock", null)},
];
 */

function SimpleSorter(parser, moveToEnd){
    
    var lines = new Array();
    parser.convertLines(function(line) {
        if(SimpleSorter.isMatching(line, moveToEnd)) {
            lines.push(line);
            return null;
        }
        return line;
    });
    for(var line in lines) {
        parser.addLine(lines[line]);
    }
    return parser;
}
SimpleSorter.isPatternMatching = function(value, pattern) {
    return ((value == null) || (pattern == null) || (value == pattern));
};

SimpleSorter.isMatching = function(line, filter) {
    return SimpleSorter.isPatternMatching(line.type, filter.type) && SimpleSorter.isPatternMatching(line.node, filter.node) && SimpleSorter.isPatternMatching(line.prop, filter.prop) && SimpleSorter.isPatternMatching(line.value, filter.value);
};
/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
// This function converts the version string into a number to make it comparable
// According to the LWR_UID driver, versions are like:
// [uint8_t].[uint8_t].[uint8_t]b[uint8_t]
function versionToNumber(versionString){
    var pattern = /(\d+)\.(\d+)\.(\d+)b(\d+)/;
    var numbers = pattern.exec(versionString);
    if(numbers == null){
        throw "ERROR: Wrong version number in converter script";
    }
    
    var ret  = Number(numbers[1] * 256 * 256 * 256);
    ret     += Number(numbers[2] * 256 * 256);
    ret     += Number(numbers[3] * 256);
    ret     += Number(numbers[3]);
    return ret;
}

function isConversionNeeded(actualVersion, convertedVersion) {
    return versionToNumber(actualVersion) < versionToNumber(convertedVersion);
}

function replaceAt(string, index, character) {
    return string.substr(0, index) + character + string.substr(index+character.length);
};

function importJs(file) {
    $.getScript(file, function(){
        alert(file + "Script loaded but not necessarily executed.");
    });
};


function convert(restoreData, version, revision, partNumber ) {
        var partNumInt = Number(replaceAt(partNumber, 4, '0')); //mask th dvigear flag
        // Decide which script is to be used
        var deviceDescriptor = null;
        switch(partNumInt) {
            case 91540019: //DP_TPS_TX210
            case 91540017: //HDMI_TPS_TX210
            case 91540015: //DVI_HDCP_TPS_TX210
                deviceDescriptor = {inputVideoNum: 1,outputVideoNum: 2,inputAudioNum: 1,outputAudioNum: 2};
                break;
            case 91540018: //DP_TPS_TX220
            case 91540011: //HDMI_TPS_TX220
            case 91540010: //DVI_HDCP_TPS_TX220
                deviceDescriptor = {inputVideoNum: 1,outputVideoNum: 2,inputAudioNum: 2,outputAudioNum: 2};
                break;
            case 91540007: //SW4_TPS_TX240
            case 91540065: //SW4_TPS_TX240_Plus
                deviceDescriptor = {inputVideoNum: 4,outputVideoNum: 2,inputAudioNum: 5,outputAudioNum: 2};
                break;
            default:
                throw "Device is not supported";
        }
        
        var parser = new Parser(restoreData);
        
        var CommonConversionScript = [
            //v1.2.0b1
            {version: "1.2.0b1", search: LW3line.property("/EDID/F/F120", null, null) , replace: null},
            
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/NETWORK", "HttpPort", null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/NETWORK", "Lw2Port", null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/NETWORK", "Lw3Port", null) , replace: null},
            {version: "1.2.0b1", search: LW3line.method("/MANAGEMENT/NETWORK", "reset", null) , replace: LW3line.method("/MANAGEMENT/NETWORK", "factoryDefaults", null)},
            
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/STATUS", "BootExtFirmware", null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/STATUS", "BootVs100Firmware", null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/STATUS", "DpChipFirmware", null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/STATUS", "Vs100Firmware", null) , replace: null},
                        
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/TASK", null, null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/TASK/CONNECTION", null, null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/TASK/CONNECTIONHANDLER", null, null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/TASK/CONNECTIONUSER", null, null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/TASK/HTTP", null, null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/TASK/Main", null, null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/TASK/UITASK", null, null) , replace: null},
                        
            {version: "1.2.0b1", search: LW3line.method("/MANAGEMENT/UI", "reset", null) , replace: LW3line.method("/MANAGEMENT/UI", "factoryDefaults", null)},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/UI", "ControlLock", "false") , replace: LW3line.property("/MANAGEMENT/UI", "ControlLock", "0")},
            {version: "1.2.0b1", search: LW3line.property("/MANAGEMENT/UI", "ControlLock", "true") , replace: LW3line.property("/MANAGEMENT/UI", "ControlLock", "1")},
                        
            {version: "1.2.0b1", search: LW3line.property("/MEDIA/AUDIO/I5", "Volume", null) , replace: null},
            
            {version: "1.2.0b1", search: LW3line.property(  "/REMOTE/D1",   "DeviceDescription", null) ,
                                  replace: LW3line.property("/REMOTE/D1", "RemoteDeviceDescription", null)},
                                  
            
            {version: "1.2.0b1", search: LW3line.property("/SYS/MB",         "AdditionalFirmwareVersions", null) ,
                                  replace: LW3line.property("/MANAGEMENT/UID", "AdditionalFirmwareVersions", null)},

            {version: "1.2.0b1", search: LW3line.property("/SYS/MB",         "AddonHwVersions", null) ,
                                  replace: LW3line.property("/MANAGEMENT/UID", "AddonHwVersions", null)},
 
            {version: "1.2.0b1", search: LW3line.property("/SYS/MB",         "DeviceLabel", null) ,
                                  replace: LW3line.property("/MANAGEMENT/UID", "DeviceLabel", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property("/SYS/MB",         "HwVersion", null) ,
                                  replace: LW3line.property("/MANAGEMENT/UID", "HwVersion", null)},

            {version: "1.2.0b1", search: LW3line.property("/SYS/MB",         "MacAddress", null) ,
                                  replace: LW3line.property("/MANAGEMENT/UID", "MacAddress", null)},

            {version: "1.2.0b1", search: LW3line.property("/SYS/MB",         "BootIpAddress", null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/SYS/MB",         "DeviceName", null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/SYS/MB",         "ManufacturerName", null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/SYS/MB",         "PartNumber", null) , replace: null},
            {version: "1.2.0b1", search: LW3line.property("/SYS/MB",         "SerialNumber", null) , replace: null},
            
            {version: "1.2.0b1", search: LW3line.property("/SYS/MB/BUTTONS",           "VideoSelectPressed", null) ,
                                  replace: LW3line.property("/MANAGEMENT/UI/BUTTONS/B1", "Clicked", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property("/SYS/MB/BUTTONS",           "AudioSelectPressed", null) ,
                                  replace: LW3line.property("/MANAGEMENT/UI/BUTTONS/B2", "Clicked", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property("/SYS/MB/BUTTONS",           "ShowMePressed", null) ,
                                  replace: LW3line.property("/MANAGEMENT/UI/BUTTONS/B3", "Clicked", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "Connected", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "Connected", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "SignalPresent", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "SignalPresent", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "SignalType", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "SignalType", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "Layout", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "Layout", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "SamplingFreq", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "SamplingFreq", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "ChannelStatus", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "ChannelStatus", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "AudioIf", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "AudioIf", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "PortId", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "PortId", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "PortDirection", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "PortDirection", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "PortUiCode", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "PortUiCode", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "Text", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "Text", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "GroupName", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "GroupName", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/DP/DPAUDIO",   "factoryDefaults", null) ,
                                  replace: LW3line.property("/SYS/MB/DP/AUDIOIN1", "factoryDefaults", null)},

            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "AudioIf", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "AudioIf", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "Balance", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "Balance", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "Connected", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "Connected", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "Gain", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "Gain", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "GroupName", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "GroupName", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "Layout", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "Layout", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "PortDirection", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "PortDirection", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "PortId", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "PortId", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "PortUiCode", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "PortUiCode", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "SamplingFreq", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "SamplingFreq", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "SignalPresent", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "SignalPresent", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "SignalType", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "SignalType", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "Text", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "Text", null)},
                                  
            {version: "1.2.0b1", search: LW3line.property("/SYS/MB/HDMI/ANALOGIN", "Volume", null) , replace: null},
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/HDMI/ANALOGIN",   "factoryDefaults", null) ,
                                  replace: LW3line.property("/SYS/MB/ADAUIN/ANALOGIN", "factoryDefaults", null)},
                                  
                                  
            {version: "1.2.0b1", search: LW3line.property(  "/SYS/MB/TPS/REMOTE",   "DeviceDescription", null) ,
                                  replace: LW3line.property("/SYS/MB/TPS/REMOTE", "RemoteDeviceDescription", null)},
                                  
        
        ];
                
        parser = TPSconvertv110(parser, version, deviceDescriptor)
        
        parser = SimpleScriptConverter(parser, version, CommonConversionScript);
        
        parser = EventManagerConverter(parser, version, revision, partNumber);
        
        parser = SimpleSorter(parser, new LW3line("/MANAGEMENT/NETWORK", null, null));
        
        return parser.output();
};


function TPSconvertv110(parser, version, deviceDescriptor ) {

        // Do EventManager conversion
        parser = EventManagerv1to2(parser, version, "1.1.0b0");

        // Do crosspoint settings conversion
        var TPSLayerConversion = [
            {layer:"VIDEO", inputNum: deviceDescriptor.inputVideoNum, outputNum: deviceDescriptor.outputVideoNum, inputString: "I", outputString: "O"},
            {layer:"AUDIO", inputNum: deviceDescriptor.inputAudioNum, outputNum: deviceDescriptor.outputAudioNum, inputString: "I", outputString: "O"}
        ];

        parser = Crosspointv1to2(parser, TPSLayerConversion, version, "1.1.0b0");

        // Do simple find-and-replace conversion
        var TPSConversionScriptv110 = [
            {version: "1.1.0b0", search: LW3line.property("/SYS/MB", "ButtonLock", null) , replace: LW3line.property("/MANAGEMENT/UI", "ControlLock", null)},
            {version: "1.1.0b0", search: LW3line.property("/SYS/MB/TPS/VS100OUT", "tpsModeSetting", null) , replace: LW3line.property("/SYS/MB/TPS/REMOTE", "tpsModeSetting", null)},
            {version: "1.1.0b0", search: LW3line.property("/SYS/MB/TPS/VS100OUT", "pwr5vMode", null) , replace: LW3line.property("/SYS/MB/TPS/VS100OUT", "Pwr5vMode", null)},
            {version: "1.1.0b0", search: new LW3line("/SYS/MB/ETH/LINK", null, null) , replace: new LW3line("/SYS/MB/TPS/ETH", null, null)},
            {version: "1.1.0b0", search: new LW3line("/SYS/MB/RS232/LINK", null, null) , replace: new LW3line("/SYS/MB/TPS/RS232", null, null)},
            {version: "1.1.0b0", search: new LW3line("/SYS/MB/CECIR", null, null) , replace: null},
            {version: "1.1.0b0", search: LW3line.property("/SYS/MB/HDMI/LOCALOUT", "PreviewEnable", null) , replace: null},
            {version: "1.1.0b0", search: LW3line.property("/SYS/MB/TPS/VS100OUT", "PreviewEnable", null) , replace: null},
            {version: "1.1.0b0", search: LW3line.method(null, "SendMessage", null) , replace: LW3line.method(null, "sendMessage", null)},
            {version: "1.1.0b0", search: LW3line.method(null, "SendText", null) , replace: LW3line.method(null, "sendText", null)},
            {version: "1.1.0b0", search: LW3line.method(null, "SendBinaryMessage", null) , replace: LW3line.method(null, "sendBinaryMessage", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "LinkState", null) , replace: LW3line.property("/REMOTE/D1", "LinkState", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "tpsMode", null) , replace: LW3line.property("/REMOTE/D1", "tpsMode", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "connectedDevice", null) , replace: LW3line.property("/REMOTE/D1", "ConnectedDevice", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "DeviceId", null) , replace: LW3line.property("/REMOTE/D1", "DeviceId", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "VendorId", null) , replace: LW3line.property("/REMOTE/D1", "VendorId", null)},
            {version: "1.1.0b0", search: LW3line.property("/MEDIA/VIDEO/O1", "tpsModeSetting", null) , replace: LW3line.property("/REMOTE/D1", "tpsModeSetting", null)},
            {version: "1.1.0b0", search: new LW3line("/SYS/MB/UMXDP{1}", null, null) , replace: new LW3line("/SYS/MB/DP{1}", null, null)},
        ];
        parser = SimpleScriptConverter(parser, version, TPSConversionScriptv110);
        
        return parser;
};

/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/*
var TPSLayerConversion = [
    {layer: "VIDEO", inputNum: 2, outputNum: 2, inputString: "I", outputString: "O"},
    {layer: "AUDIO", inputNum: 2, outputNum: 2, inputString: "I", outputString: "O"},
];
 */

function Crosspointv1to2_layer(parser, converter) {
    parser.convertLines(function(line) {
        if(line.node.startsWith("/MEDIA/"+ converter.layer)) {
            if(line.node.search("XP") !== -1) {
                line.node="/MEDIA/"+ converter.layer + "/XP";
                if(line.prop === "setSourcePortPriority") {
                    line.prop = "setAutoselectionPriority";
                    var pattern = /P(\d):(\d)/;
                    var parameters = line.value.split(";");
                    var value = "";
                    // parameters = [P0:1, P2:1, ...]
                    for(var outputIndex = 1; outputIndex <= converter.outputNum; outputIndex++){
                        for(var j in parameters){
                            if(Number(j) !== 0 || outputIndex !== 1)
                                value += ";"
                            var x = pattern.exec(parameters[j]);
                            var portNum = x[1];
                            var prio = x[2];
                            value += converter.inputString + portNum + "\\(" + converter.outputString + outputIndex + "\\):" + prio;
                        }
                    }
                    line.value = value;
                } else if(line.prop === "setDestinationPortAutoselect") {
                    var ports = line.value.split(";");
                    var value = "";
                    for(var k in ports){
                        // ports[k] = P5:DPB
                        if(Number(k) !== 0)
                            value += ";";
                        var x = /P(\d):(.+)/.exec(ports[k]);
                        var portNum = Number(x[1]);
                        var text = x[2];
                        value += converter.outputString + (portNum - converter.inputNum) + ":";
                        if(text.charAt(0) === "D")
                            value += "D";
                        else
                            value += text.charAt(0) + text.charAt(1);
                    }
                    line.value = value;
                }
            } else { //check if its port, need to convert from P
                var match = /([A-Z\/]+)P(\d)$/g.exec(line.node);
                if(match != null) {
                    var portNum = Number(match[2]);
                    if(portNum <= converter.inputNum) { //if its input
                        line.node = match[1] + converter.inputString + portNum;
                    } else { //if its output
                        line.node = match[1] + converter.outputString + (portNum - converter.inputNum);
                    }
                }
            }
        }
        return line;
    });
    return parser;
};
function Crosspointv1to2(parser, layerConversion, actualVersion, v2version){
    if(!isConversionNeeded(actualVersion, v2version)) {
        return parser;
    }
    for(var converter in layerConversion) {
        parser = Crosspointv1to2_layer(parser, layerConversion[converter]);
    }
    return parser;
};

